import { select, scaleOrdinal, schemeCategory10 } from 'd3';
import { observeResize } from '@curran/responsive-axes';
import { one } from './one';
//import data from './religionByCountryTop20.csv';
import data from './mangademotime.csv';
import { stackedBarChart } from './stackedBarChart';
import { colorLegend } from './colorLegend';

// Parse the data
const shortNames = {
  'Democratic Republic of the Congo': 'DR Congo',
};
for (const d of data) {
  d.population = +d.population / 1000000;
  d.country = shortNames[d.country] || d.country;
}

//const colorValue = (d) => d.religion;
const colorValue = (d) => d.Demographic;
const hoveredStrokeWidth = 1;
const hoveredStrokeColor = 'black';

export const main = (container, { state, setState }) => {
  const dimensions = observeResize({
    state,
    setState,
    container,
  });

  if (dimensions === null) return;

  const { width, height } = dimensions;
  const { hoveredColorValue } = state;

  const setHoveredColorValue = (hoveredColorValue) => {
    setState((state) => ({ ...state, hoveredColorValue }));
  };

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(schemeCategory10);

  const stackedBarChartG = one(
    svg,
    'g',
    'stacked-bar-chart',
  );
  const colorLegendG = one(svg, 'g', 'color-legend').attr(
    'transform',
    `translate(${width - 170}, 20)`,
  );

  stackedBarChartG.call(stackedBarChart, {
    width,
    height,
    data,
    xValue: (d) => d.population,
    xAxisLabelText: 'Population (Millions)',
    xAxisLabelOffset: 37,
    yValue: (d) => d.country,
    yAxisLabelText: 'Country',
    yAxisLabelOffset: 21,
    colorValue,
    colorScale,
    hoveredColorValue,
    hoveredStrokeWidth,
    hoveredStrokeColor,
    marginTop: 10,
    marginBottom: 47,
    marginLeft: 98,
    marginRight: 10,
  });

  colorLegendG.call(colorLegend, {
    colorScale,
    hoveredColorValue,
    setHoveredColorValue,
    hoveredStrokeWidth,
    hoveredStrokeColor,
  });
};
